#!/usr/bin/python -t
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2005 Dan Williams and Red Hat, Inc.
#

import os
import shutil
import fcntl
import threading
import time
import popen2


class lighttpdManager(threading.Thread):
    def __init__(self, conffile, hostname, port, basedir, exit_if_dead, key_and_cert):
        self._stop = False
        self._pobj = None
        self._conffile = conffile
        self._port = port
        self._hostname = hostname
        self._basedir = basedir
        self._key_and_cert = key_and_cert
        self._exit_if_dead = exit_if_dead
        threading.Thread.__init__(self)
        if not self._setup():
            os._exit(1)
        self._start_server()

    def _setup(self):
        if not os.path.exists(self._basedir):
            os.makedirs(self._basedir)
        if self._key_and_cert and not os.access(self._key_and_cert, os.R_OK):
            print "%s does not exist or is not readable." % self._key_and_cert
            return False
        if not os.path.exists(os.path.dirname(self._conffile)):
            os.makedirs(os.path.dirname(self._conffile))

        cnf = '''
server.chroot              = "%s"
server.document-root       = "/"
mimetype.assign            = (
  ".rpm"          =>      "application/x-rpm",
  ".log"          =>      "text/plain"
                             )
server.port                = %d
server.bind                = "%s"
server.username            = "lighttpd"
server.groupname           = "lighttpd"
server.dir-listing         = "disable"
''' % (self._basedir, self._port, self._hostname)

        if self._key_and_cert:
            cnf = cnf + '''
ssl.engine                 = "enable"
ssl.pemfile                = "%s"
''' % (self._key_and_cert)

        f = open(self._conffile, "w")
        f.write(cnf)
        f.close()
        return True

    def _start_server(self):
        cmd = "/usr/sbin/lighttpd -f %s -D" % self._conffile
        self._pobj = popen2.Popen4(cmd=cmd, bufsize=1024)
        fcntl.fcntl(self._pobj.fromchild.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)

    def run(self):
        while not self._stop:
            if self._pobj:
                st = self._pobj.poll()
                # Restart it if it fails
                if st >= 0:
                    if self._exit_if_dead:
                        print "HTTP server failed!  exiting..."
                        os._exit(1)
                    else:
                        print "HTTP server failed, trying to restart..."
                        print "----------------------------------------"
                        for line in self._pobj.fromchild.readlines():
                            print line
                        print "----------------------------------------\n"
                        del self._pobj
                        if not self._setup():
                            os._exit(1)
                        self._start_server()
            time.sleep(5)

        if self._pobj:
            try:
                os.kill(self._pobj.pid, 15)
                os.remove(self._conffile)
            except:
                pass

    def stop(self):
        self._stop = True
